############################################
# March, 2019
# Contain Simulation Code for Low-Dimensional Case With and Without Sparsity
# Also Compare it with heavy-tailed error and Data Contaminated Models
############################################
library(MASS)
library(rrpack)
source("/home/ktan/ktan/matrixregression/RobustMatrix.R")
source("/home/ktan/ktan/matrixregression/r4.R")

#############################################
# Simulation for matrix regression
# with rank 1 A and rank 2 A
# Classial setting for all other robust methods
# Non-sparse 
# We consider error with 
# t-distribution dof 1
# normal
# log-normal distribution
# tau >= sqrt(n/(3log(pq)))
# round(sqrt(n/3/log(p*q))+0.5)
############################################
mregsim <- function(n,p,q,seed,scenario=1,sparse=0,errortype=1,percentcont=0,loss="huber"){

	set.seed(seed)	
	A <- matrix(0,p,q)
	if(scenario==1 && sparse==0){
		p1<- rbinom(p,1,1/2);		p1[which(p1==0)] <- -1
		q1<- rbinom(q,1,1/2);		q1[which(q1==0)] <- -1
		a1 <- runif(p,0.5,1)*p1;		a2 <- runif(q,0.5,1)*q1
		A <- a1%*%t(a2)
	}
	
	if(scenario==2 && sparse==0){
		p1 <- rbinom(p,1,1/2); 		p1[which(p1==0)] <- -1
		p2 <- rbinom(p,1,1/2);		p2[which(p2==0)] <- -1 
		q1 <- rbinom(q,1,1/2); 		q1[which(q1==0)] <- -1
		q2 <- rbinom(q,1,1/2); 		q2[which(q2==0)] <- -1				
		a11 <- runif(p,0.5,1)*p1;		a12 <- runif(q,0.5,1)*q1
		a21 <- runif(p,0.5,1)*p1;		a22 <- runif(q,0.5,1)*q1
		A <- a11%*%t(a12)+a21%*%t(a22)
	}
	
	if(scenario==1 && sparse!=0){
		A[1:5,1:5] <-  1
	}
	
	if(scenario==2 && sparse!=0){
		A[1:5,1:5] <- 1;		A[3:5,3:5] <- 2
		A[6:7,6:7] <- 1
		A[3:5,6:7] <- 1;		A[6:7,3:5] <- 1
	}
	
	
	
	# Generate AR-1 type covariance for X
	Sigma <- matrix(0.5,p,p)
	tmpmat <- matrix(0,p,p)
	for(i in 1:(p-1)){
		tmpmat[i,i:p] <- c(0:(length(i:p)-1))
	}
	tmpmat = tmpmat+t(tmpmat)
	Sigma <- Sigma^tmpmat	

	X <- mvrnorm(n=n,mu=rep(0,p),Sigma=Sigma)
	X <- X/max(abs(X))
	Xtest <- mvrnorm(n=n,mu=rep(0,p),Sigma=Sigma)
	Xtest <- Xtest/max(abs(Xtest))	
	
	if(errortype==1){
		E <- matrix(rnorm(n*q,0,2),n,q)
		E <- E-mean(E)	
		Etest <- matrix(rnorm(n*q,0,2),n,q)
		Etest <- Etest-mean(Etest)			
	}	
	else if(errortype==2){
		E <- matrix(rt(n*q,1.5),n,q)
		E <- E-mean(E)
		Etest <- matrix(rt(n*q,1.5),n,q)
		Etest <- Etest-mean(Etest)			
		
	}	
	else if(errortype==3){
		E <- matrix(rlnorm(n*q,sdlog=1.2),n,q)
		E <- E-mean(E)
		Etest <- matrix(rlnorm(n*q,sdlog=1.2),n,q)
		Etest <- Etest-mean(Etest)					
		
	}		
		
	Y <- X%*%A+E
	Ytest <- Xtest%*%A+Etest
	
	if(percentcont!=0){		
		index <- sample(n*q,percentcont*0.01*n*q)
		Y[index] <- runif(length(index),min=10,max=20)
		index2 <- sample(n*q,percentcont*0.01*n*q)
		Ytest[index2] <- runif(length(index2),min=10,max=20)				
	}
	
	
	
	
# Robust Reduced Rank Regression by She and Chen 2018 
print("r4")
	lambdas <- seq(0.1,5,by=0.1)*sqrt(log(p*q)/n)
	a<-cv.rrrr(X,Y,scenario,lambdas,nfold=5,seed=1)
	lambda<-lambdas[which(apply(a$cv.error,2,mean)==min(apply(a$cv.error,2,mean)))]
	res <- rrrr(Y,X,nrank=scenario,lambda,1e-5,init.B0=init.B0,init.C0=init.C0)
	r4mse <- sum((res$B0-A)^2)
	r4pred <- sum((Xtest%*%res$B0-Xtest%*%A)^2)
	
print("r3")
# Reduced Rank Regression with Nuclear Norm
	r3A1 <- rrr(Y,X,penaltySVD="ann",ic.type="GIC",maxrank=scenario)$coef
	r3A2 <- rrr(Y,X,penaltySVD="ann",ic.type="AIC",maxrank=scenario)$coef
	r3A3 <- rrr(Y,X,penaltySVD="ann",ic.type="BIC",maxrank=scenario)$coef
	r3A4 <- rrr(Y,X,penaltySVD="ann",ic.type="BICP",maxrank=scenario)$coef
	r3A5 <- rrr(Y,X,penaltySVD="ann",ic.type="GCV",maxrank=scenario)$coef
					
	r3mse <- min(sum((r3A1-A)^2),sum((r3A2-A)^2),sum((r3A3-A)^2),sum((r3A4-A)^2),sum((r3A5-A)^2))
	r3pred <- min(sum((Xtest%*%r3A1-Xtest%*%A)^2),sum((Xtest%*%r3A2-Xtest%*%A)^2),sum((Xtest%*%r3A3-Xtest%*%A)^2),sum((Xtest%*%r3A4-Xtest%*%A)^2),sum((Xtest%*%r3A5-Xtest%*%A)^2))

	print("r4ridge")
# Reduced Rank Regression with Ridge
	lambdas <- seq(1,60,by=1)
	msetemp <- NULL; 	predtemp <- NULL
	for(lambda in lambdas){
		res <- rrs.fit(Y,X,nrank=scenario,lambda=lambda)
		msetemp <- c(msetemp,sum((res$coef-A)^2))
	}
	lambda <- lambdas[which(msetemp==min(msetemp))]
	res <- rrs.fit(Y,X,nrank=scenario,lambda=lambda)		
	rrridgemse <- sum((res$coef-A)^2)
	rrridgepred <- sum((Xtest%*%res$coef-Xtest%*%A)^2)
	
print("our proposal with squared error loss")	
# Our method with squared error loss

	lambdas <- seq(0.1,8,by=0.03)*sqrt(log(p*q)/n)
	tau <- 100000
	if(sparse==0){
		gamma <- 0
	}	
	if(sparse!=0){
		gamma <- c(2.5,3,3.5,4)
	}
	cvres <- NULL
	for(gam in gamma){
		cvres <- cbind(cvres,apply(cv.robust(X,Y,lambdas,gam,nfold=5,loss="huber",seed=seed,tau=tau)$cv.error,2,mean))
	}
	
	matmin <- function(mat){
		row <- which(mat == min(mat))[1]%%nrow(mat)
		column <- which(mat == min(mat))[1]%/%nrow(mat)+1		
		return(c(row,column))
	}
	
	if(length(gamma)==1){
		lambda <- lambdas[which(cvres==min(cvres))]
		gamma <- 0
		res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	}
	
	if(length(gamma)>1){
		tmp <- matmin(cvres)
		lambda <- lambdas[tmp[1]]
		gamma <- gamma[tmp[2]]
		res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	}
	
	oursqmse <- sum((res$A-A)^2)
	oursqpred <- sum((Xtest%*%res$A-Xtest%*%A)^2)

print("our proposal with huber loss")
cvres <- NULL	
# Our method with Huber loss function
	lambdas <- seq(0.1,6,by=0.03)*sqrt(log(p*q)/n)
	tauvalues <- seq(0.4,1.5,by=0.08)*sqrt(n/log(p*q))
	if(sparse==0){
		gamma <- 0	
	for(tautemp in tauvalues){	
		cvres <- cbind(cvres,apply(cv.robust(X,Y,lambdas,0,nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	}
	tmp <- matmin(cvres)
	lambda <- lambdas[tmp[1]]
	tau <- tauvalues[tmp[2]]
	res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	ourhbmse <- sum((res$A-A)^2)
	ourhbpred <- sum((Xtest%*%res$A-Xtest%*%A)^2)
	}	

# Our method with Huber loss function for sparse scenario	
# Consider multiple gamma for sparsity
	if(sparse!=0){
		gamma <- c(2.5,3,3.5,4)
	
	cvres <- cvres1 <- cvres2 <- cvres3 <- NULL

	for(tautemp in tauvalues){	
	cvres <- cbind(cvres,apply(cv.robust(X,Y,lambdas,gamma[1],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	cvres1 <- cbind(cvres1,apply(cv.robust(X,Y,lambdas,gamma[2],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	cvres2 <- cbind(cvres2,apply(cv.robust(X,Y,lambdas,gamma[3],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	cvres3 <- cbind(cvres3,apply(cv.robust(X,Y,lambdas,gamma[4],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	print(tautemp)
	}
	tmp <- c(min(cvres),min(cvres1),min(cvres2),min(cvres3))
	a <- which(tmp==min(tmp))[1]
	tmp1 <- matmin(cvres);	tmp2 <- matmin(cvres1); 	tmp3 <- matmin(cvres2)	
	tmp4 <- matmin(cvres3)
	tmp <- rbind(tmp1,tmp2,tmp3,tmp4)[a,]
	lambda <- lambdas[tmp[1]]
	tau <- tauvalues[tmp[2]]
	gamma <- gamma[a]
	res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	ourhbmse <- sum((res$A-A)^2)
	ourhbpred <- sum((Xtest%*%res$A-Xtest%*%A)^2)
	
}

return(list(r3mse=r3mse,r3pred=r3pred,r4mse=r4mse,r4pred=r4pred, rrridgemse= rrridgemse, rrridgepred=rrridgepred, oursqmse= oursqmse,oursqpred= oursqpred,ourhbmse= ourhbmse, ourhbpred= ourhbpred,tau=tau))
}




#############################################
# Simulation for matrix regression in high dimensions
############################################
runhighd <- function(n,p,q,seed,scenario=1,errortype=1,percentcont=0,loss="huber"){

	set.seed(seed)	
	A <- matrix(0,p,q)
		
	if(scenario==1){
		A[1:5,1:5] <-  1
	}
	
	if(scenario==2){
		A[1:5,1:5] <- 1;		A[3:5,3:5] <- 2
		A[6:7,6:7] <- 1
		A[3:5,6:7] <- 1;		A[6:7,3:5] <- 1
	}
	
	
	
	# Generate AR-1 type covariance for X
	Sigma <- matrix(0.5,p,p)
	tmpmat <- matrix(0,p,p)
	for(i in 1:(p-1)){
		tmpmat[i,i:p] <- c(0:(length(i:p)-1))
	}
	tmpmat = tmpmat+t(tmpmat)
	Sigma <- Sigma^tmpmat	

	X <- mvrnorm(n=n,mu=rep(0,p),Sigma=Sigma)
	X <- X/max(abs(X))
	Xtest <- mvrnorm(n=n,mu=rep(0,p),Sigma=Sigma)
	Xtest <- Xtest/max(abs(Xtest))	
	
	if(errortype==1){
		E <- matrix(rnorm(n*q,0,2),n,q)
		E <- E-mean(E)	
		Etest <- matrix(rnorm(n*q,0,2),n,q)
		Etest <- Etest-mean(Etest)			
	}	
	else if(errortype==2){
		E <- matrix(rt(n*q,1.5),n,q)
		E <- E-mean(E)
		Etest <- matrix(rt(n*q,1.5),n,q)
		Etest <- Etest-mean(Etest)			
		
	}	
	else if(errortype==3){
		E <- matrix(rlnorm(n*q,sdlog=1.2),n,q)
		E <- E-mean(E)
		Etest <- matrix(rlnorm(n*q,sdlog=1.2),n,q)
		Etest <- Etest-mean(Etest)					
		
	}		
		
	Y <- X%*%A+E
	Ytest <- Xtest%*%A+Etest
	
	if(percentcont!=0){		
		index <- sample(n*q,percentcont*0.01*n*q)
		Y[index] <- runif(length(index),min=10,max=20)
		index2 <- sample(n*q,percentcont*0.01*n*q)
		Ytest[index2] <- runif(length(index2),min=10,max=20)				
	}
	
	
	
	print("our proposal with squared error loss")	

# Our method with squared error loss
	lambdas <- seq(0.1,8,by=0.03)*sqrt(log(p*q)/n)
	tau <- 100000
	if(sparse==0){
		gamma <- 0
	}	
	if(sparse!=0){
		gamma <- c(2.5,3,3.5,4)
	}
	cvres <- NULL
	for(gam in gamma){
		cvres <- cbind(cvres,apply(cv.robust(X,Y,lambdas,gam,nfold=5,loss="huber",seed=seed,tau=tau)$cv.error,2,mean))
	}
	
	matmin <- function(mat){
		row <- (which(mat == min(mat))[1]-1)%%nrow(mat)
		column <- (which(mat == min(mat))[1]-1)%/%nrow(mat)+1		
		return(c(row,column))
	}
	
	if(length(gamma)==1){
		lambda <- lambdas[which(cvres==min(cvres))]
		gamma <- 0
		res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	}
	
	if(length(gamma)>1){
		tmp <- matmin(cvres)
		lambda <- lambdas[tmp[1]]
		gamma <- gamma[tmp[2]]
		res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	}
	
	oursqmse <- sum((res$A-A)^2)
	oursqpred <- sum((Xtest%*%res$A-Xtest%*%A)^2)
	oursqspec <- sum(abs(res$Z)<=1e-5 & abs(A)==0)/sum(A==0)
	oursqsens <- sum(abs(res$Z)>=1e-5 & A!=0)/sum(A!=0)	
		
		
		
print("our proposal with huber loss")
cvres <- NULL	
# Our method with Huber loss function
	lambdas <- seq(0.1,5,by=0.03)*sqrt(log(p*q)/n)
	tauvalues <- seq(0.4,1.5,by=0.08)*sqrt(n/log(p*q))
	if(sparse==0){
		gamma <- 0	
	for(tautemp in tauvalues){	
		cvres <- cbind(cvres,apply(cv.robust(X,Y,lambdas,0,nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	}
	tmp <- matmin(cvres)
	lambda <- lambdas[tmp[1]]
	tau <- tauvalues[tmp[2]]
	res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	ourhbmse <- sum((res$A-A)^2)
	ourhbpred <- sum((Xtest%*%res$A-Xtest%*%A)^2)
	}	

# Our method with Huber loss function for sparse scenario	
# Consider multiple gamma for sparsity
	if(sparse!=0){
		gamma <- c(2.5,3,3.5,4)
	
	cvres <- cvres1 <- cvres2 <- cvres3 <- NULL

	for(tautemp in tauvalues){	
	cvres <- cbind(cvres,apply(cv.robust(X,Y,lambdas,gamma[1],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	cvres1 <- cbind(cvres1,apply(cv.robust(X,Y,lambdas,gamma[2],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	cvres2 <- cbind(cvres2,apply(cv.robust(X,Y,lambdas,gamma[3],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	cvres3 <- cbind(cvres3,apply(cv.robust(X,Y,lambdas,gamma[4],nfold=5,loss="huber",seed=seed,tau=tautemp)$cv.error,2,mean))
	print(tautemp)
	}
	tmp <- c(min(cvres),min(cvres1),min(cvres2),min(cvres3))
	a <- which(tmp==min(tmp))[1]
	tmp1 <- matmin(cvres);	tmp2 <- matmin(cvres1); 	tmp3 <- matmin(cvres2)	
	tmp4 <- matmin(cvres3)
	tmp <- rbind(tmp1,tmp2,tmp3,tmp4)[a,]
	lambda <- lambdas[tmp[1]]
	tau <- tauvalues[tmp[2]]
	gamma <- gamma[a]
	res <- robustmat(Y,X,lambda,gamma*lambda,tau,convergence=1e-10,maxiter=1000,rho=10/n)
	ourhbmse <- sum((res$A-A)^2)
	ourhbpred <- sum((Xtest%*%res$A-Xtest%*%A)^2)
	ourhbspec <- sum(abs(res$Z)<=1e-5 & abs(A)==0)/sum(A==0)
	ourhbsens <- sum(abs(res$Z)>=1e-5 & A!=0)/sum(A!=0)	
	
}

return(list(oursqmse= oursqmse,oursqpred= oursqpred,ourhbmse= ourhbmse, oursqspec=oursqspec,oursqsens=oursqsens,ourhbpred= ourhbpred,ourhbspec=ourhbspec,ourhbsens=ourhbsens,tau=tau))
}





